// ********************************************************************************
// Servo Library https://github.com/arduino-libraries/Servo.git //
// SEB : 30/03/2022 - Pilotage 2 servo Moteur avec radiocommande 
// DUMBO RC 2.4Ghz projet JAD Fauteuil pour spectacle
// Attention : fonctionnement aléatoire si radio commande emetterice non allumée.
// ********************************************************************************

#include <Servo.h> 

// Mode debug : True
#define DEBUG true

// Utilisation des channels 1 et 2 pour le pilotage 
#define CH1_PIN 2
#define CH2_PIN 3

//  Vitesse max des servo moteur = 90 +90 =180 ou bien 90-90 = 0
// plus on s'éloigne du point millieu,plus le moteur accèlère pour retrouver sa position

// plage de fonctionnement de la vitesse linéaire
#define MAX_SPEED 90 
#define MAX_LINEAR_SPEED 90 

// plage de fonctionnement des vitesses de rotation
#define MIN_SPEED_ROTATION 20
#define MAX_SPEED_ROTATION 40

#define MAX_SPEED_OFFSET 20  // minimum pour démarrer

#define SERVO_PIN_LEFT  4
#define SERVO_PIN_RIGHT 5

#define LED_ARDUINO_ON      11
#define LED_SECURITY_SWITCH 10

Servo servoLeftWheel, servoRightWheel;   

// TEST recepteur Radicocommande Dumbox6 channel (channel 1 et 2 )
int ch1 =0;
int ch2 =0;

int rotationSpeed = 0;
int rotationDir = 0;

int linearDir = 0;
int linearSpeed = 0;

int speedLeftMotor = 0;
int speedRightMotor = 0;

  
void setup() {
  Serial.begin(57600);

  // tentative en pull up pour avoir un etat stable sur le singal radio commande meme si emetteur éteint.  
  pinMode(CH1_PIN, INPUT_PULLUP);
  pinMode(CH2_PIN, INPUT_PULLUP);

  // Test pour régler le point milieu et régler les servos à l'arrêt (à 90° les servos ne doivent pas tourner) 
  servoLeftWheel.write(90);
  servoRightWheel.write(90);

  servoLeftWheel.attach(SERVO_PIN_LEFT);  
  servoRightWheel.attach(SERVO_PIN_RIGHT); 

  // LED indicateur bon fonctionnement arduino
  pinMode(LED_ARDUINO_ON,OUTPUT);
  digitalWrite(LED_ARDUINO_ON, HIGH); 

  // HARDWARE non réalisé : TO DO
  // Allumer si les 2 switch de détection de poid sur le fauteuil sont fermés
  // Par defaut alumée, s'éteind si ok, sinon verifier les switchs...
  pinMode(LED_SECURITY_SWITCH,OUTPUT);
  digitalWrite(LED_SECURITY_SWITCH, HIGH); 
}

// PROG Principal
void loop() {

  if (DEBUG) Serial.print(ch2);Serial.print(" - ");Serial.println(ch1);

  // Decodage réception de la commande de direction :  rot horaire 867 -repos 1260 - rot anti horaire : 1840
  ch1 = pulseIn(CH1_PIN, HIGH);
 
  if (ch1>1200 && ch1<1300) {
    rotationDir=0;     // arrêt
    rotationSpeed=0;
   } else if (ch1<1200) {
    rotationDir=1; // tourne à gauche
    rotationSpeed=map(ch1, 860, 1200, MAX_SPEED_ROTATION, 0);
  } else if (ch1>1300) {
    rotationDir=-1; // tourne à droite
    rotationSpeed=map(ch1, 1300, 1840, 0, MAX_SPEED_ROTATION);  
  }

  // ajustement de la vitesse du moteur proportionnelle pour virage plus ou moins rapide
  
  // accelerateur : position repos : 1270   =>accelere => 1930   =>recul : 860
  ch2 = pulseIn(CH2_PIN, HIGH);

  if (ch2>1200 && ch2<1300) {
    linearDir=0;
    linearSpeed=0;
  } else if (ch2<1200) {
    linearDir=-1; // RECULE
    linearSpeed=map(ch2, 860, 1200,MAX_LINEAR_SPEED, MAX_SPEED_OFFSET );
  } else if (ch2>1300) {
    linearDir=1; // AVANCE
    linearSpeed=map(ch2, 1300, 1930, MAX_SPEED_OFFSET, MAX_LINEAR_SPEED);  
  }


  // calcul les vitesse des moteurs droite et gauche
  // Par defaut même vitesse sur les 2 moteurs avant correction pour gérer la rotation en mouvement
   speedLeftMotor = linearSpeed;
   speedRightMotor = linearSpeed;
    
   // on joue sur une reduction de vitesse sur l'une des roues pour gerer un déviation droite ou gauche
   if (rotationDir<0) speedLeftMotor = linearSpeed-abs(rotationSpeed);
   if (rotationDir>0) speedRightMotor = linearSpeed-abs(rotationSpeed);

   // On s'assure que la vitesse est bien dans les bornes de tolérances définies
   speedLeftMotor=min(MAX_SPEED,speedLeftMotor);
   speedRightMotor=min(MAX_SPEED,speedRightMotor);

  // rotation sur place
  if (linearDir==0) {
    if (rotationDir==-1) turnRight();
    if (rotationDir==1) turnLeft();
  }
  
  // Arrêt
  if (rotationDir==0 && linearDir==0) {
    moveStop();    
  }

  // Avance 
  if (linearDir==1) {
    moveForward();
  }
  
  // RECULE
  if (linearDir==-1) {
    moveBackward();
  }
 
 delay(10); 
}

// met les servos à vitesse 0 et detach pour qu'ils ne forcent pas.
void moveStop() {
  servoLeftWheel.write(90);
  servoRightWheel.write(90);
  delay(10);
  servoLeftWheel.detach();  
  servoRightWheel.detach(); 

} 
  
void moveForward() {
  if (DEBUG) Serial.println("Forward L - R ::");Serial.print(speedLeftMotor);Serial.print(" - ");Serial.println(speedRightMotor);
  servoLeftWheel.write(90-speedLeftMotor);
  servoRightWheel.write(90+speedRightMotor);
    
  servoLeftWheel.attach(SERVO_PIN_LEFT);  
  servoRightWheel.attach(SERVO_PIN_RIGHT); 

}

void moveBackward() {
  if (DEBUG) Serial.println("Backward L - R ::");Serial.print(speedLeftMotor);Serial.print(" - ");Serial.println(speedRightMotor);
 
  servoLeftWheel.write(90+speedLeftMotor);
  servoRightWheel.write(90-speedRightMotor);

  servoLeftWheel.attach(SERVO_PIN_LEFT);  
  servoRightWheel.attach(SERVO_PIN_RIGHT); 
}  



void turnRight() {

  rotationSpeed=abs(rotationSpeed);
  rotationSpeed=max(MIN_SPEED_ROTATION, rotationSpeed);
 
  servoLeftWheel.write(90+rotationSpeed);
  servoRightWheel.write(90+rotationSpeed);

  servoLeftWheel.attach(SERVO_PIN_LEFT);  
  servoRightWheel.attach(SERVO_PIN_RIGHT); 
    
} 
 
void turnLeft() {
  
  rotationSpeed=abs(rotationSpeed);
  rotationSpeed=max(MIN_SPEED_ROTATION, rotationSpeed);
  
  servoLeftWheel.write(90-rotationSpeed);
  servoRightWheel.write(90-rotationSpeed);  

  servoLeftWheel.attach(SERVO_PIN_LEFT);  
  servoRightWheel.attach(SERVO_PIN_RIGHT); 

}  
